/** \file
 * Executable which runs hellocomm::helloapp.
 * hello-comm-world is a very simple executable to run the hellocomm::helloapp
 * example skeleton mpi application within the SST/Macro simulator.
 */

#include <sstmac/backends/native/driver.h>
#include <unistd.h> // getopt
#include "helloapp.h"

using namespace sstmac;
using namespace sstmac::hw;
using namespace sstmac::sw;
using namespace sstmac::native;

int
main()
{
  //-------------------------------------
  // Section 1. Construct parameters, if not reading from a file
  //-------------------------------------
  command_parameters::ptr params = command_parameters::construct();
  params->add_param("node_num", "2");
  params->add_param("node_cores", "2");
  params->add_param("node_mem_latency", "1e-9");
  params->add_param("node_mem_bandwidth", "125MBps");
  params->add_param("node_frequency", "1e9");
  params->add_param("node_name", "simple");
  params->add_param("nic_name", "simple");

  params->add_param("network_name", "exclusive");
  params->add_param("network_latency", "380ns");
  params->add_param("network_bandwidth", "524Mbps");

  params->add_param("topology_name", "ring");
  params->add_param("topology_geometry", "2");

  params->add_param("launch_name", "instant");
  params->add_param("launch_indexing", "block");
  params->add_param("launch_allocation", "firstavailable");

  params->add_param("launch_app1", "myhelloworld");
  params->add_param("launch_app1_size", "2");
  params->add_param("launch_app1_start", "0ms");

  params->add_param("app_message", "Why, hello world!");

  //-------------------------------------
  // Section 2. Add a custom application not yet in the skeleton factory
  //-------------------------------------

  // Construct an application object
  sstmac::sw::appid aid(0);
  sstmac::sw::taskid tid(0);
  sstmac::sw::softwareid sid(aid, tid);
  app::ptr application(hellocomm::helloapp::construct(sid));

  sstmac::sw::skeletonfactory::apps["myhelloworld"] = application;

  //-------------------------------------
  // Section 3. Set any debug output
  //-------------------------------------
  logger::set_user_param("<debug> app");

  //-------------------------------------
  // Section 4. Run the thing until it's done (time = -1)
  //-------------------------------------
  run(params, timestamp(-1), false);

}
