/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/backends/native/components/node/degradednode.h>

#include <sstmac/hardware/processor/simpleprocessor-config.h>
#include <sstmac/hardware/processor/instructionprocessor-config.h>
#include <sstmac/backends/native/components/node/simplenode.h>

#include <sstmac/common/errors.h>
#include <sstmac/software/services/launch/machine_descriptor.h>

namespace sstmac { namespace native {


 void
  degradednode::initialize(nodeid::ptr my_id, const sstmac::sw::service::ptr &the_launcher)
  {
    node::initialize(my_id, the_launcher);

    double tcycle = 1.0 / (freq_ / factor_);

    timestamp tflop(tcycle * 10); // 10 clock cycles for a floating point
    timestamp tmemop(memory_latency_.sec() + 1.0 / (memory_bandwidth_ / factor_));


    sstmac::hw::simpleprocessor_config::ptr cfg =
        sstmac::hw::simpleprocessor_config::construct(cores_, freq_, 10, 30);

    compute_sched_ = sstmac::hw::computescheduler::construct(cores_, preempt_);

    proc_model_ = sstmac::hw::simpleprocessor::construct(cfg, msg_handler_, compute_sched_);


    sstmac::hw::instructionprocessor_config::ptr pcfg =
        sstmac::hw::instructionprocessor_config::construct(cores_,
            INST_FLOP_LABEL, tflop, INST_MEMOP_LABEL, tmemop, INST_MEMOP_LOCAL_LABEL,
            tmemop, INST_MEMOP_RAND_LABEL, tmemop);

    proc_inst_ = sstmac::hw::instructionprocessor::construct(pcfg, msg_handler_,
        compute_sched_);

  }


} }
