/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_DEGRADEDNODE_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_DEGRADEDNODE_H_INCLUDED

#include <sstmac/backends/native/components/node/simplenode.h>

#include <sstmac/hardware/processor/simpleprocessor.h>
#include <sstmac/hardware/processor/instructionprocessor.h>
#include <sstmac/hardware/processor/computescheduler.h>
#include <sstmac/backends/native/components/nic/networkinterface.h>
#include <sstmac/backends/native/components/node/node.h>
#include <sstmac/common/nocopy.h>

#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/units.h>

#include <map>

//#include <sstmac/processor/processor_config.h>

namespace sstmac
{
  namespace native
  {

    /**
     * A simple node that is degraded, cutting its performance by a factor
     */
    class degradednode : public simplenode, virtual public nocopy
    {

    protected:
      degradednode(double fac, const timestamp &memory_latency,
          double memory_bandwidth, double frequency, int n_core, timestamp pre) :
          simplenode(memory_latency, memory_bandwidth, frequency, n_core, pre), factor_(
              fac)
      {

      }

      double factor_;

    public:
      typedef boost::intrusive_ptr<degradednode> ptr;


      static ptr
      construct(sim_parameters::ptr params, double factor)
      {
        int n_cores = params->get_int_param("node_cores");
        timestamp lat = params->get_time_param("node_mem_latency");
        bool errorflag = false;
        double bw = get_bandwidth(
            params->get_param("node_mem_bandwidth").c_str(), errorflag);
        double freq = params->get_double_param("node_frequency");

        timestamp pre(-1);
        if (params->has_param("node_preemption"))
          {
            pre = params->get_time_param("node_preemption");
          }

        return ptr(new degradednode(factor, lat, bw, freq, n_cores, pre));
      }

      virtual void
      initialize(nodeid::ptr my_id,
          const sstmac::sw::service::ptr &the_launcher);

      virtual
      ~degradednode()
      {

      }

    };

  }
} // end of namespace sstmac

#endif
