/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/backends/native/components/node/node.h>
#include <sstmac/backends/native/components/nic/networkinterface.h>

#include <sstmac/software/process/operatingsystem.h>



namespace sstmac { namespace native {

  debug node::dbg_;

  node::node()
  {
    sstmac::sw::hardware_descriptor::ptr desc =
        sstmac::sw::hardware_descriptor::construct(1);
    os_ = sstmac::sw::operatingsystem::construct(desc);

    if(!dbg_.is_real()){
        dbg_ = debug("<debug> node");
        logger::register_logger(&dbg_);
    }
  }

  void node::set_environment(const sstmac::sw::environment::ptr &env,
      sstmac::sw::appid aid)
  {

    os_->set_environment(env, aid);
  }

  void node::start_application(sstmac::sw::app::ptr &a,
      const sim_parameters::ptr &params)
  {
    os_->add_application(a, params);
    os_->start_apps();
  }

  void node::initialize(nodeid::ptr my_id, const sstmac::sw::service::ptr &the_launcher)
  {
    nodeid_ = my_id;
    msg_handler_ = message_handler::construct(ptr(this));

    launcher_ = boost::dynamic_pointer_cast<sstmac::sw::launcher>(the_launcher);
    os_->register_ser(launcher_);

  }

  void node::handle(const sst_message::ptr &msg)
  {
    msg_handler_->handle(msg);
  }

  } } // end of namespace sstmac

