/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_DRIVER_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_DRIVER_H_INCLUDED

#include <sstmac/common/driver_util.h>

#include <sstmac/backends/native/manager.h>
#include <sstmac/backends/native/managerdegraded.h>

//node models
#include <sstmac/backends/native/components/node/simplenode.h>

//nic models
#include <sstmac/backends/native/components/nic/simplenic.h>

//event managers
#include <sstmac/backends/native/eventmap.h>

namespace sstmac { namespace native
{

  using namespace std;

  eventmanager::ptr
  run(const sim_parameters::ptr &params, timestamp until, bool print)
  {

    bool errorflag = false;

    interconnect::ptr fab = construct_circuit(params);


    manager::ptr mgr;


    if(!params->has_param("manager") || params->get_param("manager") == "simple"){
        mgr = manager::construct();
    }else if(params->get_param("manager") == "degraded"){
        mgr = manager_degraded::construct();
    }

    mgr->set_interconnect(fab);

    mgr->add_nodes(params);

    indexstrategy::ptr idx = construct_indexing(
        params->get_param("launch_indexing"), params, errorflag);
    allocationstrategy::ptr all = construct_allocation(
        params->get_param("launch_allocation"), params, fab->get_topology(),
        errorflag);

    std::list<launchinfo::ptr> apps = construct_launchers(params);

    instantlaunch::ptr job;

    if (params->get_param("launch_name") == "instant")
      {
        job = instantlaunch::construct(apps, all, idx);
      }

    timestamp runtime = mgr->run(job, until);

    debug dbg("<debug> sstmac | sim | simulation");

    if (print)
      {
        dbg << "runtime\n";
        dbg << runtime.sec() << "\n";
      }

    return mgr->get_eventmanager();
  }

} }

#endif
