/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/backends/native/sstevent.h>

namespace sstmac { namespace native
{



sst_event::sst_event(const sst_message::ptr &msg,
                const eventhandler::ptr &hand) :
                to_deliver_(msg), handler_(hand)
{

}

sst_event::~sst_event()
{

}

void sst_event::execute()
{
        handler_->handle(to_deliver_);
}

sst_event::ptr sst_event::construct(
                const sst_message::ptr &msg,
                const eventhandler::ptr &hand)
{
        sst_event::ptr ret(new sst_event(msg, hand));
        return ret;
}

std::string sst_event::to_string() const
{
        std::stringstream ss;
        ss << "sst_event(handler=" << handler_->to_string() << ",  msg = "
                        << (to_deliver_ ? to_deliver_->to_string() : "null") << ")";
        return ss.str();
}



} } // end of namespace sstmac
