/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_OMNETPP_NETWORKMODELS_CYCLEACCURATE_CYCLEACCURATESWITCH_H_INCLUDED
#define SSTMAC_BACKENDS_OMNETPP_NETWORKMODELS_CYCLEACCURATE_CYCLEACCURATESWITCH_H_INCLUDED

#include <map>
#include <queue>
#include <networkmodels/rackswitch.h>

#include "network_m.h"

namespace sstmac { namespace omnetpp {


#define ROUTER_DELAY 0.0000001
  //#define ROUTER_DELAY 0

  class cycleaccurateswitch : public rackswitch
  {

  protected:

    virtual void
    handleMessage(cMessage* cmg);
    virtual void
    initialize();
    virtual void
    finish();

    std::map<cGate*, int> inports;
    std::map<int, cGate*> outports;

    std::map<int, int> credits_;

    int numVC_;
    int inBuffSize_;
    int outBuffSize_;

    double clockFreq_;

    simtime_t arbLatency_;
    simtime_t switchLatency_;
    double switchBandwidth_;

    std::map<int, cPacketQueue*> inbuffs_;
    std::map<int, cPacketQueue*> outbuffs_;
    std::map<int, cMessage*> outBuffMsgs_;
    std::map<cMessage*, int> revOutBuffMsgs_;

    cMessage* clockmsg_;
    simtime_t clockper_;

    // ---- arbitration ------ //
    void
    arbitrate();

    std::map<int, simtime_t> inports_free_;
    std::map<int, simtime_t> outports_free_; //of the xbar, not the whole switch

    int arb_inport_;
    std::map<int, int> arb_vc_;
    std::map<int, int> num_in_buff_;

    std::map<int, std::queue<cMessage*>*> pending_creds_up_;

  };

} }
#endif
