/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_OMNETPP_NETWORKMODELS_RACKSWITCH_H_INCLUDED
#define SSTMAC_BACKENDS_OMNETPP_NETWORKMODELS_RACKSWITCH_H_INCLUDED

#include <omnetpp.h>

#include <networkmodels/routing/router.h>

#include <sstmac/common/debug.h>

#include <string>

namespace sstmac { namespace omnetpp {


  class rackswitch: public cSimpleModule {
  public:
    rackswitch() {
      if(!dbg_.is_real()){
           dbg_ = debug("<debug> switch | network");
           logger::register_logger(&dbg_);
       }
    };

    virtual ~rackswitch();

  protected:

    void init(omnetpp_address::ptr me, int npr, int nrc, std::string top, int tp1, int tp2, int tp3) {
      myAddr = me;
      nodesPerRack = npr;
      numRackConnections = nrc;

      instantiateRouter(top, tp1, tp2, tp3);
    }

    virtual void instantiateRouter(std::string top, int p1, int p2, int p3);

    omnetpp_address::ptr myAddr;
    router* router_;

    int nodesPerRack;
    int numRackConnections;

    static sstmac::debug dbg_;

    /*static std::map<cChannel*, rackswitch*> channel_map1_;
        static std::map<cChannel*, rackswitch*> channel_map2_;

        static std::map<cChannel*, int> channel_vals1_;
        static std::map<cChannel*, int> channel_vals2_;

        void
        register_channel(cChannel* ch, rackswitch* rs);
        void
        update_channel_display(cChannel* ch, rackswitch* rs, double frac);
     */
    bool displayConnections;
  };

} }
#endif
