/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/backends/omnetpp/node/processor/medium_serializer.h>
#include <sstmac/backends/omnetpp/omnetppaddress.h>

namespace sstmac { namespace omnetpp {

  medium_serializer::~medium_serializer()
  {

  }

  boost::intrusive_ptr<nodeaddress>
  medium_serializer::get_deser_addr(){
    return omnetpp_address::construct();
  }

  void
  medium_serializer::pack(int i)
  {
    buffer->pack(i);
  }

  void
  medium_serializer::unpack(int &i)
  {
    buffer->unpack(i);
  }

  void
  medium_serializer::pack(long int i)
  {
    buffer->pack(i);
  }

  void
  medium_serializer::unpack(long int &i)
  {
    buffer->unpack(i);
  }

  void
  medium_serializer::pack(long long int i)
  {
    buffer->pack(i);
  }

  void
  medium_serializer::unpack(long long int &i)
  {
    buffer->unpack(i);
  }

  void
  medium_serializer::pack(size_t i)
  {
    buffer->pack(i);
  }

  void
  medium_serializer::unpack(size_t &i)
  {
    buffer->unpack(i);
  }

  void
  medium_serializer::pack(std::string i)
  {
    opp_string os(i);
    buffer->pack(os);
  }
  void
  medium_serializer::unpack(std::string &i)
  {
    opp_string os;
    buffer->unpack(os);
    i.assign(os.c_str());
  }

  void
  medium_serializer::pack(timestamp i)
  {
    buffer->pack(i.sec());
  }

  void
  medium_serializer::unpack(timestamp &i)
  {
    double val;
    buffer->unpack(val);
    i = timestamp(val);
  }

  void
  medium_serializer::pack(bool i)
  {
    buffer->pack(i);
  }

  void
  medium_serializer::unpack(bool &i)
  {
    buffer->unpack(i);
  }



  void
  medium_serializer::pack(richpair<int, int> i)
  {
    buffer->pack(i.first);
    buffer->pack(i.second);
  }

  void
  medium_serializer::unpack(richpair<int, int> &i)
  {
    int f, s;
    buffer->unpack(f);
    buffer->unpack(s);
    i = std::make_pair(f, s);
  }



} }
