/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_OMNETPP_OMNETPP_PARAMS_H_INCLUDED
#define SSTMAC_BACKENDS_OMNETPP_OMNETPP_PARAMS_H_INCLUDED

#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/errors.h>
#include <omnetpp.h>

namespace sstmac
{
  namespace omnetpp
  {

    class omnetpp_params : public sim_parameters
    {
    protected:
      omnetpp_params(cSimpleModule* p) :
          parent_(p)
      {

      }

    public:
      typedef boost::intrusive_ptr<omnetpp_params> ptr;

      static ptr
      construct(cSimpleModule* p)
      {
        return ptr(new omnetpp_params(p));
      }

      virtual std::string
      get_param(std::string key) const
      {
        return parent_->par(key.c_str());
      }

      virtual int
      get_int_param(std::string key) const
      {
        return parent_->par(key.c_str());
      }

      virtual long
      get_long_param(std::string key) const
      {
        return parent_->par(key.c_str());
      }

      virtual double
      get_double_param(std::string key) const
      {
        return parent_->par(key.c_str());
      }

      virtual bool
      has_param(std::string key) const
      {
        return parent_->hasPar(key.c_str());
      }

      virtual void
      add_param(std::string key, std::string val)
      {
        throw unimplementederror("omnetpp_params::add_param - don't call this");
      }

      virtual void
      add_param_override(std::string key, std::string val)
      {
        throw unimplementederror(
            "omnetpp_params::add_param_override - don't call this");

      }

      virtual void
      combine_into(sim_parameters::ptr sp)
      {
        throw unimplementederror(
            "omnetpp_params::combine_into - don't call this");
      }

      virtual std::string
      to_string() const
      {
        return "omnetpp_parameters";
      }

      virtual
      ~omnetpp_params()
      {
      }

    protected:
      cSimpleModule* parent_;

    };

  }
}

#endif
