/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/eventmodule.h>

#include <sstmac/common/eventhandler.h>
#include <sstmac/common/eventmanager.h>
#include <sstmac/common/messages/sst_message.h>

namespace sstmac {

  void eventmodule::schedule(const timestamp &t,
      const eventhandler::ptr &handler,
      const sst_message::ptr &msg)
  {
    if (eventman_ == 0)
      {
        std::cout << "my type: " << typeid(*this).name() << std::endl;
        throw nullerror("eventmodule::schedule - null event manager!");
      }
    eventman_->schedule(t, handler, msg);
  }

  void eventmodule::set_eventmanager(const eventmanager::ptr &m)
  {
    eventman_ = m;

    std::list<stat_logger::ptr>::iterator it, end = need_reg_.end();
    for(it = need_reg_.begin(); it != end; it++){
        eventman_->register_stat(*it);
    }
    need_reg_.clear();
  }

  timestamp eventmodule::now()
  {
    if (eventman_ == 0)
      {
        std::cout << "my type: " << typeid(*this).name() << std::endl;
        throw nullerror("eventmodule::now - null event manager!");
      }
    return eventman_->now();
  }
} // end of namespace sstmac
