

/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_SIM_PARAMETERS_H_INCLUDED
#define SSTMAC_COMMON_SIM_PARAMETERS_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/units.h>

#include <sstmac/common/basicstringtokenizer.h>

namespace sstmac{
  class sim_parameters : public ptr_type {
  protected:
    sim_parameters()  :
      dbg_("<debug> parameters | params") {}

  public:
    typedef boost::intrusive_ptr<sim_parameters> ptr;

    virtual std::string
    get_param(std::string key) const = 0;

    virtual void
    add_param(std::string key, std::string val) = 0;

    virtual void
    add_param_override(std::string key, std::string val) = 0;

    virtual void
    combine_into(sim_parameters::ptr sp) = 0;

    static std::pair<std::string, std::string>
     parse_param(std::string p)
     {
       std::string lhs = p.substr(0, p.find("="));
       std::string rhs = p.substr(p.find("=") + 1);

       if(lhs.find("**.") != std::string::npos)
           lhs = lhs.substr(3);

       lhs = sstmac::trim_str(lhs);
       rhs = sstmac::trim_str(rhs);

       return std::make_pair<std::string, std::string>(lhs, rhs);
     }

    virtual bool
    has_param(std::string key) const = 0;

    virtual int
    get_int_param(std::string key) const{
      std::string v = get_param(key);
      return atoi(v.c_str());
    }

    virtual long
    get_long_param(std::string key) const {
        std::string v = get_param(key);
        return atol(v.c_str());
    }

    virtual double
    get_double_param(std::string key) const{
      std::string v = get_param(key);
      return atof(v.c_str());
    }

    timestamp
    get_time_param(std::string key) const {
      bool error = false;
      return get_timestamp(get_param(key).c_str(), error);

    }

  protected:
    sstmac::debug dbg_;

  };
}

#endif
