/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_STATS_STATS_COMMON_H_INCLUDED
#define SSTMAC_COMMON_STATS_STATS_COMMON_H_INCLUDED

#include <sstmac/common/errors.h>
#include <sstmac/common/stats/stat_logger.h>
#include <iostream>
#include <fstream>
#include <map>
#include <list>

#include <sstmac/common/timestamp.h>


namespace sstmac {

  /**
   * this stat_logger class just adds up all the records given to it,
   * and outputs it to file when destructed.  values can be associated with
   * string keys for storing multiple things
   */
  class stat_sum : public stat_logger
  {
  public:
    typedef stat_sum* ptr;

  protected:


    stat_sum(std::string namesig, std::string filename) :
      stat_logger(namesig, filename)
    {
    }

  public:
    static ptr
    construct(std::string namesig, std::string filename){
      return new stat_sum(namesig, filename);
    }

    virtual void simulation_finished(timestamp end){
      if(is_active() && check_open()){
          myfile << "stat_sum(" << signature_ << ") \n";
          std::map<std::string, double>::iterator it, end = val_.end();
          for(it = val_.begin(); it != end; it++){
              myfile << it->first << " ," << it->second << "\n";
          }
          myfile.close();
      }
    }

    virtual
    ~stat_sum()
    {
    }


    virtual void record(double i, std::string key = "default"){
      if(val_.find(key) == val_.end())
        val_[key] = 0;
      val_[key] += i;
    }

  protected:

    std::map<std::string, double> val_;

  };

}

#endif
