/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_CONGESTION_CIRCUIT_TRANSFER_FINISH_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_CONGESTION_CIRCUIT_TRANSFER_FINISH_H_INCLUDED

#include <sstmac/hardware/network/congestion/circuit-transfer.h>
#include <sstmac/hardware/network/congestion/circuit-edge.h>

namespace sstmac { namespace hw  {

  /**
   * Release the edges in a circuit simulator.
   */
  class circuit::transfer::finish: public eventhandler {
    /// The parent circuit.
    circuit::ptr parent_;
    /// The edges we will be releasing.
    typedef std::vector<int> edges_t;
    edges_t edges_;

  protected:
    /// Hi.
    finish(const circuit::ptr &parent,
        const std::vector<int> &edges) :
          parent_(parent), edges_(edges) {
    }



  public:
    typedef boost::intrusive_ptr<finish> ptr;

    /// Hi.
    static finish::ptr construct(
        const circuit::ptr &parent,
        const std::vector<int> &edges) {
      return finish::ptr(new finish(parent, edges));
    }

    /// Bye.
    virtual ~finish() throw () {
    }

    /// All done.
    virtual void handle(const sst_message::ptr &msg) {

      for (edges_t::const_iterator it = edges_.begin(); it != edges_.end(); ++it) {

          parent_->edge_.at(*it).release();
      }


    }

    virtual std::string to_string() const {
      return "circuit::transfer-finish";
    }
  };

} } // end of namespace sstmac

#endif
