/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_CONGESTION_CONSTANTBANDWIDTH_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_CONGESTION_CONSTANTBANDWIDTH_H_INCLUDED

#include <sstmac/hardware/network/congestion/bandwidthmodel.h>
#include <sstmac/common/ptr_type.h>

namespace sstmac {

  /**
   * The simplest bandwidth model.  All transfers are assumed to have
   * the same constant bandwidth.
   */
  class constantbandwidth : public bandwidthmodel {
    /// The bandwidth we dole out.
    bandwidth bandwidth_;
    
    /// And off we go.
    constantbandwidth(bandwidth bw) :
      bandwidth_(bw)
    {}

  public:
    typedef boost::intrusive_ptr<constantbandwidth> ptr;

    /// May we offer you some constant bandwidth?
    /// \param bw  bandwidth in bytes/ps
    static constantbandwidth::ptr construct(bandwidth bw) {
      return constantbandwidth::ptr(new constantbandwidth(bw));
    }

    /// No more Mr. Nice Constant Bandwidth.
    virtual ~constantbandwidth() {}

    /// What sort of bandwidth are we talking about?
    /// \return bandwidth in bytes/ps.
    virtual bandwidth bw(data) const {
      return bandwidth_;
    }
  };

} } // end of namespace sstmac.

#endif
