/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_CONGESTION_LATENCYMODEL_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_CONGESTION_LATENCYMODEL_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/timestamp.h>
#include <sstmac/hardware/network/congestion/data.h>
#include <stdint.h>

namespace sstmac { namespace hw 
  {

  /**
   * A latencymodel provides facilities for estimating the latency
   * associated with a data transfer.  This is a pure abstract base type.
   */
  class latencymodel : public ptr_type
  {
  protected:
    latencymodel() {}
  public:
    virtual std::string to_string() const{
      return "latencymodel";
    }

    typedef boost::intrusive_ptr<latencymodel> ptr;

    /// Goodbye.
    virtual ~latencymodel()
    {
    }

    /// Estimate the latency for a data transfer of the given amount of data.
    virtual timestamp latency(data payload) const = 0;


  };

  } } // end of namespace sstmac

#endif
