/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_CONGESTION_SHAREDCIRCUIT_STARTER_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_CONGESTION_SHAREDCIRCUIT_STARTER_H_INCLUDED

#include <sstmac/hardware/network/congestion/sharedcircuit-stopper.h>
#include <vector>

namespace sstmac { namespace hw  {

  /**
   * Handle firing off a transfer request for sharedcircuit.
   */
  class sharedcircuit::starter: public eventhandler {
    flower::ptr flower_;
    stopper::ptr stopper_;
    int64_t bytes_;
    timestamp recv_delay_;
    std::vector<edge_id_t> edge_id_;
    eventhandler::ptr sender_;
    eventhandler::ptr recver_;
    sharedcircuit::ptr parent_;

  protected:
    /// A new beginning.
    starter(const sharedcircuit::ptr &par,
        const flower::ptr &flow,
        const sharedcircuit::stopper::ptr &stop,
        int64_t bytes, const timestamp &recv_delay,
        const std::vector<edge_id_t> edge_id,
        const eventhandler::ptr &sender,
        const eventhandler::ptr &recver) :
          flower_(flow), stopper_(stop), bytes_(bytes), recv_delay_(recv_delay),
          edge_id_(edge_id), sender_(sender), recver_(recver),
          parent_(par) {
    }

  public:
    typedef boost::intrusive_ptr<starter> ptr;

    /// Start me up (kachunk boom).
    static starter::ptr construct(
        const sharedcircuit::ptr &par,
        const flower::ptr &flow,
        const sharedcircuit::stopper::ptr &stop,
        int64_t bytes, const timestamp &recv_delay,
        const std::vector<edge_id_t> edge_id,
        const eventhandler::ptr &sender,
        const eventhandler::ptr &recver) {
      return starter::ptr(
          new starter(par, flow, stop, bytes, recv_delay, edge_id,
              sender, recver));
    }

    /// I'll never stop.
    virtual ~starter() throw () {
    }

    /// Our time has come.
    virtual void handle(const sst_message::ptr &msg) {
      flow_id_t fid = flower_->start_flow(parent_->now(), edge_id_, bytes_);
      stopper_->add(fid, recv_delay_, msg, sender_, recver_);
      flow_id_t stopflow = flower_->next_stopping_flow();
      timestamp stoptime = flower_->next_stopping_time();
      stopper_->update(stopflow, stoptime);

      if(parent_->energystat_->is_active()){
          parent_->energystat_->record(parent_->now(),
              edge_id_.size() * parent_->pwr_config_->get_rtr_hop_per_byte() * bytes_, stoptime - parent_->now());
          bandwidthstat_->record(parent_->now(), bytes_, stoptime - parent_->now());

      }
    }

    virtual std::string to_string() const {
      return "sharedcircuit-starter";
    }
  };

} } // end of namespace sstmac

#endif
