/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


#include <sstmac/hardware/network/congestion/transfer_time.h>

namespace sstmac { namespace hw  {

  ////////////////////////////////////////////////////////////////////////
  // transfer_time

  transfer_time::~transfer_time()
  {
  }

  ////////////////////////////////////////////////////////////////////////
  // latency_rbandwidth_time

  transfer_time::ptr
  latency_rbandwidth_time::construct(double latency,
                                     double rbandwidth)
  {
    latency_rbandwidth_time *self = new latency_rbandwidth_time;

    self->latency = latency;
    self->rbandwidth = rbandwidth;

    transfer_time::ptr ret(self);
    return ret;
  }

  double
  latency_rbandwidth_time::time(int64_t nbyte)
  {
    return latency + rbandwidth * nbyte;
  }

  void
  latency_rbandwidth_time::print(std::ostream &o)
  {
    o<<"latency_rbandwidth_time("<<latency<<","<<rbandwidth<<")";
  }

  ////////////////////////////////////////////////////////////////////////
  // constant_time

  transfer_time::ptr
  constant_time::construct(double constant)
  {
    constant_time *self = new constant_time;

    self->constant = constant;

    transfer_time::ptr ret(self);
    return ret;
  }

  double
  constant_time::time(int64_t nbyte)
  {
    return constant;
  }

  void
  constant_time::print(std::ostream &o)
  {
    o<<"constant_time("<<constant<<")";
  }

  ////////////////////////////////////////////////////////////////////////
  // rbandwidth_time

  transfer_time::ptr
  rbandwidth_time::construct(double rbandwidth)
  {
    rbandwidth_time *self = new rbandwidth_time;

    self->rbandwidth = rbandwidth;

    transfer_time::ptr ret(self);
    return ret;
  }

  double
  rbandwidth_time::time(int64_t nbyte)
  {
    return rbandwidth*nbyte;
  }

  void
  rbandwidth_time::print(std::ostream &o)
  {
    o<<"rbandwidth_time("<<rbandwidth<<")";
  }

  ////////////////////////////////////////////////////////////////////////
  // no_time

  transfer_time::ptr
  no_time::construct()
  {
    transfer_time::ptr ret(new no_time);
    return ret;
  }

  double
  no_time::time(int64_t nbyte)
  {
    return 0.0;
  }

  void
  no_time::print(std::ostream &o)
  {
    o<<"no_time()";
  }

  ////////////////////////////////////////////////////////////////////////
  // utilities

  std::ostream&
  operator<<(std::ostream&s,
             const transfer_time::ptr&t)
  {
    t->print(s);
    return s;
  }

}  } //end of namespace sstmac
