/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/common/ptr_type.h>
#include <sstmac/hardware/network/topology/composite.h>

#include <string>
#include <sstream>
#include <iostream>

namespace sstmac { namespace hw 
{
  composite::composite(const topology::ptr& n1,
      const topology::ptr& n2)
  {
    assemble(n1, n2);

    std::ostringstream ostr;
    ostr << "Composite" << "(" << n1->name() << "," << n2->name() << ")";
    name_ = ostr.str();
  }

  composite::ptr
  composite::construct(const topology::ptr& n1,
      const topology::ptr& n2)
  {
    return composite::ptr(new composite(n1, n2));
  }

  composite::~composite()
  {
  }

  std::string
  composite::name() const
  {
    return name_;
  }
} } //end of namespace sstmac
