/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

// composite.h: Concrete class for product networks.
#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_COMPOSITE_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_COMPOSITE_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/hardware/network/topology/product.h>
#include <string>

namespace sstmac { namespace hw 
{

  class composite : public product
  {
  private:
    std::string name_;

    /** The composite network contructs a new network by taking the product
     of two networks. */
  protected:
    composite(const topology::ptr& n1,
        const topology::ptr& n2);
  public:
    typedef boost::intrusive_ptr<composite> ptr;

    static composite::ptr
    construct(const topology::ptr& n1,
        const topology::ptr& n2);
    virtual
    ~composite();
    std::string
    name() const;
  };
} } //end of namespace sstmac

#endif
