/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

// edge.cc: Implementation of edges.
//
// Author: Curtis Janssen <cljanss@ca.sandia.gov>
#include <sstmac/hardware/network/topology/edge.h>
#include <sstmac/hardware/network/topology/graph.h>
#include <iostream>

using std::ostream;

namespace sstmac { namespace hw 
{

//////////////////////////////////////////////////////////////////////
// edge members and related functions

  bool
  edge::operator <(const edge &e) const
  {
    if (v1_ < e.v1_)
      return true;
    else if (v1_ == e.v1_)
      {
        if (v2_ < e.v2_)
          return true;
        else if (v2_ == e.v2_)
          {
            if (id_ < e.id_)
              return true;
          }
      }
    return false;
  }

  void
  edge::print(ostream &o) const
  {
    o << v1_ << " (" << id_ << ")-> " << v2_;
  }

  std::ostream &
  operator <<(std::ostream &o, const edge&e)
  {
    e.print(o);
    return o;
  }

//////////////////////////////////////////////////////////////////////
// lwedge members and related functions

  lwedge::lwedge(const graph &g, const vertex &v1, const vertex &v2, int id)
  {
    init(g, v1, v2, id);
  }

  lwedge::lwedge(const graph &g, const std::vector<int> &v1,
      const std::vector<int> &v2, int id)
  {
    init(g, v1, v2, id);
  }

  void
  lwedge::init(const graph &g, const vertex &v1, const vertex &v2, int id)
  {
    init(g.vertex_to_index(v1), g.vertex_to_index(v2), id);
  }

  void
  lwedge::init(const graph &g, const std::vector<int> &v1,
      const std::vector<int> &v2, int id)
  {
    vertex vertex1(v1), vertex2(v2);
    init(g.vertex_to_index(vertex1), g.vertex_to_index(vertex2), id);
  }

  void
  lwedge::print(ostream &o) const
  {
    o << v1_ << " (" << id_ << ")-> " << v2_;
  }

  std::ostream &
  operator <<(std::ostream &o, const lwedge&e)
  {
    e.print(o);
    return o;
  }

//////////////////////////////////////////////////////////////////////
// edge_index members and related functions

  std::ostream&
  operator<<(std::ostream&s, const edge_index&e)
  {
    s << e.value();
    return s;
  }

} } //end of namespace sstmac
