/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


// hypercube.h: Interface for hypercube networks.
//
// Author: Curtis Janssen <cljanss@ca.sandia.gov>

#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_HYPERCUBE_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_HYPERCUBE_H_INCLUDED

#include <sstmac/hardware/network/topology/topology.h>

namespace sstmac { namespace hw  {

  /** The hypercube network generates a network with a hypercube topology.
      It has 2^d nodes, where d is the dimention of the hypercube. Each node
      is directly connected to a switching element that connects to d other
      switches.

      <table border="1">

      <tr><td>Keyword<td>Type<td>Default<td>Description

      <tr><td><tt>d</tt><td>int<td>8<td>The dimension of the hypercube.

      </table>

  */
  class hypercube: public topology {
  private:
    int d_;
    void init_hypercube(int d);

  protected:
    //hypercube(TiXmlHandle);
    hypercube(int d);

  public:
    typedef boost::intrusive_ptr<hypercube> ptr;

    static hypercube::ptr construct(int dimension = 8);
    virtual ~hypercube();

    void shortest_path(const vertex &v1, const vertex &v2, walk &) const;
    int64_t diameter() const;
    double a() const;
    int64_t b() const;
    bool vertex_symmetric() const;
    std::string name() const;
  };

} } //end of namespace sstmac

#endif
