/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_PROCESSOR_INSTRUCTIONPROCESSOR_H_INCLUDED
#define SSTMAC_HARDWARE_PROCESSOR_INSTRUCTIONPROCESSOR_H_INCLUDED

#include <sstmac/hardware/processor/processor.h>

#include <sstmac/common/timestamp.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/common/ptr_type.h>


#include <sstmac/common/stats/power_config.h>


#include <sstmac/hardware/processor/instructionprocessor-config.h>

namespace sstmac { namespace hw 
  {



  /// A very simple processor model

  class instructionprocessor : public processor, virtual public nocopy
  {

  private:
    timestamp tflop_, tmem_, tmemloc_, tmemrnd_;
    instructionprocessor_config::ptr config_;

    //
    // Event data labels
    //

    /// Flop event data label.
    std::string flop_label_;
    /// Memory event data label.
    std::string memop_label_;
    /// Local memory operation event data label.
    std::string memop_local_label_;
    /// Random memory operation event data label.
    std::string memop_rand_label_;

  protected:

    /// Protected constructor.
    instructionprocessor(
        const instructionprocessor_config::ptr& cfg,
        const eventhandler::ptr &completion, computescheduler::ptr sched);

  public:
    typedef boost::intrusive_ptr<instructionprocessor> ptr;

    /// Goodbye.
    virtual
    ~instructionprocessor()
    {
    }

    /// Create simple processor model.
    /// \Param cfg model configuration object
    static instructionprocessor::ptr
    construct(const instructionprocessor_config::ptr& cfg,
        const eventhandler::ptr &completion, computescheduler::ptr sched);

    /// Inherited from processor.
    processor::ptr
    clone() const;

    /// Process a compute kernel.
    /// \param inst is the instructions object that will be run.
    /// \return timestamp time to process instructions.
    virtual void
    compute(const sst_message::ptr& inst);

    virtual std::string to_string() const {
      return "instructionprocessor";
    }

  };

  } } // end of namespace sstmac

#endif
