/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/actor/actorpattern.h>
#include <sstmac/common/errors.h>

namespace sstmac { namespace sw {

  /* A pattern that matches everything. */
  class actorpattern::anypattern : public actorpattern {


    /// Goodbye.
    virtual ~anypattern() {}

    /// Match.
    bool matches(const actormsg::ptr& msg) const {
      if(msg)
        return true;
      else
        throw nullerror("actorpattern::anypattern:  NULL message is not allowed.");
    }
  };
  
  // The static match for any pattern.
  actorpattern::ptr actorpattern::any_;

  // Access the any pattern matcher.
  actorpattern::ptr actorpattern::any() {
    if(! any_)
      any_ = actorpattern::ptr(new anypattern);
    return any_;
  }

} } // end of namespace sstmac.
