/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/compute/lib_compute_memmove.h>

#include <sstmac/software/process/key.h>

namespace sstmac
{
  namespace sw
  {

    using namespace std;

    debug lib_compute_memmove::dbg_;

    std::string
    lib_compute_memmove::get_lib_name()
    {
      std::stringstream ss;
      ss << "computelibmemmove" << id_.to_string();
      return ss.str();
    }

    lib_compute_memmove::lib_compute_memmove(softwareid id) :
            library(id)
    {
      if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> lib_compute_memmove | lib_memory");
          logger::register_logger(&dbg_);
        }
    }

    //
    // Build a new minimd object.
    //
    lib_compute_memmove::ptr
    lib_compute_memmove::construct(softwareid id)
    {
      lib_compute_memmove::ptr retval(new lib_compute_memmove(id));
      return retval;
    }

    lib_compute_memmove::~lib_compute_memmove()
    {

    }

    void
    lib_compute_memmove::handle(const sst_message::ptr &msg)
    {
      if (keys_.find(msg) == keys_.end())
        {
          throw libraryerror(
              "lib_compute_memmove: message arrived which is not in our key list");
        }

      SSTMAC_DEBUG << "lib_compute_memmove: received message, unblocking \n";

      key::ptr key = keys_[msg];
      os_->unblock(key);
    }

    void
    lib_compute_memmove::compute_block_access(long bytes)
    {
      if (supported())
        {
          sst_message::ptr cmsg = mem_access::construct(bytes, get_lib_name());

          key::ptr key = key::construct();
          keys_[cmsg] = key;
          SSTMAC_DEBUG << "lib_compute_memmove: executing mem access kernel \n";

          os_->execute_kernel(AMI::AMI_COMPUTE, AMI::AMI_COMP_MEMACCESS, cmsg);

          SSTMAC_DEBUG << "lib_compute_memmove: blocking \n";
          os_->block(key);

        }
      else
        {
          throw valueerror(
              "lib_compute_memmove requires the presence of a memory model");
        }
    }

  }
} //end of namespace sstmac
