/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_MEMMOVE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_MEMMOVE_H_INCLUDED

#include <sstmac/software/libraries/compute/lib_compute_inst.h>

#define BLOCK_SIZE 512

namespace sstmac
{
  namespace sw
  {

    class lib_compute_memmove : virtual public library
    {
      // ------- constructor / boost stuff -------------//

    protected:
      lib_compute_memmove(softwareid id);

    public:
      typedef boost::intrusive_ptr<lib_compute_memmove> ptr;

      virtual
      ~lib_compute_memmove();

      static lib_compute_memmove::ptr
      construct(softwareid id);
      // --------------------------------------//

      virtual bool
      supported()
      {
        return os_->kernel_supported(AMI::AMI_COMPUTE, AMI::AMI_COMP_MEMACCESS);
      }

    public:

      void
      compute_block_access(long bytes);

      virtual std::string
      get_lib_name();

      virtual void
      handle(const sst_message::ptr &msg);

    protected:

      std::map<sst_message::ptr, key::ptr> keys_;

      static debug dbg_;

      class mem_access : public sst_message
      {
      protected:
        mem_access(long bytes, std::string libn) : sst_message(libn, timestamp(0), sst_message::COMPUTE, 0),
            bytes_(bytes)
        {

        }

        long bytes_;
      public:
        typedef boost::intrusive_ptr<mem_access> ptr;

        static ptr
        construct(long bytes, std::string libn)
        {
          return ptr(new mem_access(bytes, libn));
        }

        virtual long
        get_byte_length() const
        {
          return bytes_;
        }
      };

    };

  }
} //end of namespace sstmac

#endif
