/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/compute/lib_compute_time.h>

#include <sstmac/software/process/key.h>

namespace sstmac { namespace sw
{

  debug lib_compute_time::dbg_;
  

  std::string
  lib_compute_time::get_lib_name()
  {
    std::stringstream ss;
    ss << "computelib" << id_.to_string();
    return ss.str();
  }

  lib_compute_time::lib_compute_time(softwareid id) :
      library(id)
  {
    if(!dbg_.is_real()){
         dbg_ = debug("<debug> lib_compute_time | lib_compute");
         logger::register_logger(&dbg_);
     }
  }

  //
  // Build a new minimd object.
  //
  lib_compute_time::ptr
  lib_compute_time::construct(softwareid id)
  {
    lib_compute_time::ptr retval(new lib_compute_time(id));
    // retval->set_self(retval);
    return retval;
  }

  lib_compute_time::~lib_compute_time()
  {

  }

  void
  lib_compute_time::handle(const sst_message::ptr &msg)
  {
    if (keys_.find(msg) == keys_.end())
      {
        throw libraryerror(
            "lib_compute_time: message arrived which is not in our key list");
      }

    if (dbg_.is_active())
      dbg_ << "lib_compute_time: received message, unblocking \n";

    key::ptr key = keys_[msg];
    os_->unblock(key);
  }
  void
  lib_compute_time::compute(timestamp time)
  {
    if (supported())
      {
        if(time.sec() < 0){
            throw valueerror("lib_compute_time can't compute for less than zero time");
        }
        sst_message::ptr cmsg = sst_message::construct(
            get_lib_name(), time);
        key::ptr key = key::construct();
        keys_[cmsg] = key;
        if (dbg_.is_active())
          dbg_ << "lib_compute_time: executing compute kernel \n";
        os_->execute_kernel(AMI::AMI_COMPUTE, AMI::AMI_COMP_TIME, cmsg);

        if (dbg_.is_active())
          dbg_ << "lib_compute_time: blocking \n";
        os_->block(key);
      }
    else
      {
        throw valueerror(
            "lib_compute_time requires the presence of a processor model which can compute for just an amount of time");
      }
  }

} } //end of namespace sstmac
