/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/compute/lib_sleep.h>

#include <sstmac/software/process/key.h>

namespace sstmac { namespace sw
{

  debug lib_sleep::dbg_;
  

  std::string
  lib_sleep::get_lib_name()
  {
    std::stringstream ss;
    ss << "sleeplib" << id_.to_string();
    return ss.str();
  }

  lib_sleep::lib_sleep(softwareid id) :
      library(id)
  {
    if(!dbg_.is_real()){
         dbg_ = debug("<debug> lib_sleep");
         logger::register_logger(&dbg_);
     }
  }

  //
  // Build a new minimd object.
  //
  lib_sleep::ptr
  lib_sleep::construct(softwareid id)
  {
    lib_sleep::ptr retval(new lib_sleep(id));

    return retval;
  }

  lib_sleep::~lib_sleep()
  {

  }

  void
  lib_sleep::handle(const sst_message::ptr &msg)
  {
    if (keys_.find(msg) == keys_.end())
      {
        throw libraryerror(
            "lib_sleep: message arrived which is not in our key list");
      }

    SSTMAC_DEBUG << "lib_sleep: received message, unblocking \n";

    key::ptr key = keys_[msg];
    os_->unblock(key);
  }
  void
  lib_sleep::sleep(timestamp time)
  {
    if (supported())
      {
        sst_message::ptr cmsg = sst_message::construct(
            get_lib_name(), time);
        key::ptr key = key::construct();
        keys_[cmsg] = key;
        SSTMAC_DEBUG << "lib_sleep: executing compute kernel \n";
        os_->execute_kernel(AMI::AMI_COMPUTE, AMI::AMI_COMP_SLEEP, cmsg);

        SSTMAC_DEBUG << "lib_sleep: blocking \n";
        os_->block(key);
      }
    else
      {
        throw valueerror(
            "lib_sleep requires the presence of a processor model which can sleep");
      }
  }

} } //end of namespace sstmac
