/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_QUEUE_MPIQUEUE_PROBEREQUEST_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_QUEUE_MPIQUEUE_PROBEREQUEST_H_INCLUDED

#include <sstmac/software/libraries/mpi/mpi_queue/mpiqueue.h>

namespace sstmac
{
  namespace sw
  {

    /**
     * Nested type to handle individual probe requests.
     */
    class mpiqueue::proberequest : public ptr_type
    {
      /// The parameters we will be matching on.
      mpicommid myid_;
      mpiid source_;
      mpitag tag_;
      /// The object waiting for our successful completion.
      /// If this pointer is null, nobody will be told about our raging success.
      // boost::shared_ptr<mpiprobe> completion_;
      eventhandler::ptr completion_;
      mpirequest_t key_;

    protected:
      /// Hi there.
      proberequest(const mpirequest_t key, const mpicomm &comm,
          const mpiid &source, const mpitag &tag,
          eventhandler::ptr completion);

    public:

      virtual std::string
      to_string() const
      {
        return "proberequest";
      }

      typedef boost::intrusive_ptr<proberequest> ptr;

      /// Hello.
      static proberequest::ptr
      construct(const mpirequest_t key, const mpicomm &comm,
          const mpiid &source, const mpitag &tag,
          eventhandler::ptr completion);

      /// Test whether we match a given message.
      bool
      matches(const mpimessage::ptr& message) const;

      /// Consider this request complete.
      void
      complete(const mpimessage::ptr& message);
    };

  }
} // end of namespace sstmac.

#endif
