/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_QUEUE_MPIQUEUE_SENDREQUEST_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_QUEUE_MPIQUEUE_SENDREQUEST_H_INCLUDED

#include <sstmac/software/libraries/mpi/mpi_queue/mpiqueue.h>
#include <sstmac/common/messages/messageid.h>
#include <sstmac/common/nocopy.h>
#include <sstmac/common/debug.h>

namespace sstmac { namespace sw {

  // forward declaration of important types.
  //class payload;

  /**
   * A nested type to handle individual send operations.
   */
  class mpiqueue::sendrequest : virtual public nocopy, public ptr_type {
    friend class mpiqueue;

    /// The queue.
    boost::intrusive_ptr<mpiqueue> queue_;
    /// The size at which we switch between ready-send and handshake.
    /// If ready-send is specified at construction, handshaking is not used.
    int64_t handsize_;
    int64_t envelope_;
    int64_t mintrans_;
    /// The transmission paramters.
    int count_;
    mpitype type_;
    mpiid dest_;
    mpitag tag_;
    mpicomm comm_;
    bool rsend_;
    bool ssend_;
    mpimessage::category cat_;
    int seqno_;
    messageid id_;
    /// The object that gets notified when we're done.
    eventhandler::ptr completion_;
    payload::const_ptr content_;
    /// Status flag to test whether we're waiting for an ack.
    bool pending_ack_;
    bool data_sent_;

    std::string libname_;


    mpirequest_t key_;


  protected:
    /// Hello.
    sendrequest(const mpirequest_t key, std::string libn, const mpiqueue::ptr& queue, int64_t handsize,
                int64_t envelope, int64_t mintrans, int count,
                const mpitype &type, const mpiid &dest, const mpitag& tag,
                const mpicomm &comm, bool rsend, bool ssend, 
                mpimessage::category message_type,
                int sequence_number, const messageid &id,
                const eventhandler::ptr& completion,
                const payload::const_ptr& content);





  public:

    virtual std::string to_string() const{
            return "sendrequest";
          }

    typedef boost::intrusive_ptr<sendrequest> ptr;
    /// Hi there.
    static sendrequest::ptr
    construct(const mpirequest_t key, std::string libn, const mpiqueue::ptr& queue, int64_t handsize,
              int64_t envelope, int64_t mintrans, int count,
              const mpitype &type, const mpiid &dest, const mpitag& tag,
              const mpicomm &comm, bool rsend, bool ssend, 
              mpimessage::category message_type,
              int sequence_number, const messageid &id,
              const eventhandler::ptr& completion,
              const payload::const_ptr& content);

    /// Goodbye.
    virtual ~sendrequest() throw();

    /// Go.
    void start(const mpiserver::ptr& lib);

    /// We just received an ack.
    void ack(
             const mpimessage::ptr& response,
             const mpiserver::ptr& lib);

    bool matches(const mpimessage::ptr& send_ack) const;

    /// Eventhandler completion.
   void complete(const sst_message::ptr &msg);
  };

} } // end of namespace sstmac

#endif
