/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_strategies/mpicorebarrier.h>
#include <sstmac/software/libraries/mpi/mpi_strategies/mpi_collective/engines/mpibarrierengine.h>

#include <sstmac/software/libraries/mpi/mpi_strategies/mpi_iterator/hfmiterator.h>
#include <sstmac/software/libraries/unblockhandler.h>
#include <sstmac/common/eventhandler.h>

namespace sstmac { namespace sw
{

  // Pointer types.
  

  //
  // Hello.
  //
  mpicorebarrier::mpicorebarrier()
  {
  }

  //
  // Hi.
  //
  mpicorebarrier::ptr
  mpicorebarrier::construct()
  {
    return mpicorebarrier::ptr (new mpicorebarrier);
  }

  //
  // Goodbye.
  //
  mpicorebarrier::~mpicorebarrier() throw ()
  {
  }

  //
  // Build a kernel to synchronize nodes.
  //
  void
  mpicorebarrier::execute(const mpirequest_t& thekey, const
      mpiqueue::ptr& queue, const mpitag &tag, const mpicomm &comm,
      OS_API::ptr &os
  ) const
  {
    // Here we might want to pick different iterators depending on message size.
    topologyiterator::ptr iter = hfmiterator::construct(comm.rank().id,
        comm.size().id);
    key::ptr k = thekey->get_key();
    eventhandler::ptr completion = unblockhandler::construct(os,
        k);
    mpibarrierengine::ptr engine = mpibarrierengine::construct(iter, thekey,
        queue, tag, comm, completion);
    engine->start();
  }

} } // end of namespace sstmac
