/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICOREBARRIER_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICOREBARRIER_H_INCLUDED

#include <sstmac/software/libraries/mpi/mpi_strategies/mpistrategies.h>

namespace sstmac { namespace sw {

  /**
   * Synchronize nodes.
   *
   * Performed using approximately the mechanism from mpich2-1.0.8
   * (see util/hfmiterator.h).
   */
  class mpicorebarrier : public mpibarrierstrategy {
  protected:
    /// Hello.
    mpicorebarrier();

  public:
    typedef boost::intrusive_ptr<mpicorebarrier> ptr;
    /// Hi.
    static mpicorebarrier::ptr construct();

    /// Goodbye.
    virtual ~mpicorebarrier() throw();
    
    /// Build a kernel to synchronize nodes.
    virtual void
    execute(const mpirequest_t& thekey,
          const mpiqueue::ptr& queue,
          const mpitag &tag, const mpicomm &comm,
          OS_API::ptr &os
          ) const;
  };

} } // end of namespace sstmac

#endif
