/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_strategies/mpicorescatterv.h>

#include <sstmac/software/libraries/unblockhandler.h>
#include <sstmac/software/libraries/mpi/mpi_strategies/mpi_collective/engines/mpidirectscatterv.h>

#include <sstmac/common/errors.h>
#include <sstmac/common/debug.h>

namespace sstmac { namespace sw
{

  

  //
  // Hi.
  //
  mpicorescatterv::mpicorescatterv()
  {
  }

  //
  // Hello.
  //
  mpicorescatterv::ptr
  mpicorescatterv::construct()
  {
    return mpicorescatterv::ptr(new mpicorescatterv);
  }

  //
  // Perform a fused reduce-scatterv operation
  //
  void
  mpicorescatterv::execute(const mpirequest_t& thekey,
      const mpiqueue::ptr& queue, const std::vector<int> &sendcount,
      const mpitype &sendtype, int recvcount, const mpitype &recvtype,
      const mpiid &root, const mpitag &tag, const mpicomm &comm, const std::vector<
          payload::const_ptr > &cc, OS_API::ptr &os) const
  {
    // return mpidirectscatterv::construct(thekey, queue, sendcount, sendtype,
    //                                    recvcount, recvtype, root, tag, comm,
    //                                    content);

    key::ptr k = thekey->get_key();
    eventhandler::ptr completion = unblockhandler::construct(os, k);
    mpidirectscatterv::ptr engine = mpidirectscatterv::construct(
        thekey, queue, sendcount, sendtype, recvcount, recvtype, root, tag,
        comm, cc, completion);

    engine->start();
  }

} } // end of namespace sstmac
