/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICORESSEND_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPI_STRATEGIES_MPICORESSEND_H_INCLUDED

#include <sstmac/software/libraries/mpi/mpi_strategies/mpistrategies.h>

namespace sstmac { namespace sw {

  /// Basic strategy for a "fire-and-forget" send operation.
  class mpicoressend : public mpisendstrategy {
  protected:
    mpicoressend();
  public:
    typedef boost::intrusive_ptr<mpicoressend> ptr;

    /// Hello.
    static mpicoressend::ptr construct();
    /// Goodbye.
    virtual ~mpicoressend() throw();
    /// Basic nonblocking send.  A simulator may opt to delay completion of
    /// this operation depending on message size.
    virtual void
    execute(const mpirequest_t& thekey,
          const mpiqueue::ptr& queue,
          int count, const mpitype &type, const mpiid &dest,
          const mpitag &tag, const mpicomm &comm,
          const payload::const_ptr& content,
          OS_API::ptr &os
          ) const;
  };

} } // end of namespace sstmac

#endif
