/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MPI_MPIAPP_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MPI_MPIAPP_H_INCLUDED

#include <sstmac/software/process/app.h>

#include <sstmac/software/libraries/mpi/mpi_queue/mpiqueue.h>
#include <sstmac/software/libraries/mpi/mpi_strategies/mpistrategy.h>
#include <sstmac/software/libraries/mpi/mpiserver.h>
#include <sstmac/software/libraries/mpi/mpi_comm/mpicommfactory.h>
#include <sstmac/software/libraries/mpi/mpiapi.h>

namespace sstmac { namespace sw
{

  /// Important types.
  class mpiapi;

  /**
   * A base type for applications that can run via mpi.
   * Intended to be safe for multiple inheritance (for applications
   * that use MPI and some other form of communication).
   */
  class mpiapp : public app
  {
    mpiapi::ptr mpi_;

  protected:

    mpiapp(softwareid id) :
      app(id)
    {

      mpi_ = mpiapi::construct(id);
      register_lib(mpi_);
    }

    /// Access the MPI API.
    const mpiapi::ptr&
    mpi() const
    {
      return mpi_;
    }

  public:
    typedef boost::intrusive_ptr<mpiapp> ptr;

    /// Goodbye.
    virtual
    ~mpiapp()
    {
    }

    virtual void
    kill()
    {

      if (!mpi_->finalized())
        {
          std::cerr << "mpiapi[" << mpi_->comm_world().rank().to_string()
              << "]: app destroyed before finalized" << std::endl;
        }

      app::kill();
    }

  };

} } // end of namespace sstmac.

#endif
