/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/util/lib_display.h>

#include <sstmac/software/process/key.h>

namespace sstmac { namespace sw
  {




  std::string
  lib_display::get_lib_name()
  {
    std::stringstream ss;
    ss << "displaylib" << id_.to_string();
    return ss.str();
  }

  lib_display::lib_display(softwareid id) :
          library(id)
  {

  }

  //
  // Build a new minimd object.
  //
  lib_display::ptr
  lib_display::construct(softwareid id)
  {
    lib_display::ptr retval(new lib_display(id));
    // retval->set_self(retval);
    return retval;
  }

  lib_display::~lib_display()
  {

  }

  void
  lib_display::handle(const sst_message::ptr &msg)
  {

  }

  void
  lib_display::change_border(DISPLAY_COLORS c)
  {
    if(os_->kernel_supported(AMI::AMI_DISP, AMI::AMI_DISP_BORDER)){
        sst_message::ptr changecolor = sst_message::construct("dummy",
            timestamp(0), sst_message::SST, c);
        os_->execute_kernel(AMI::AMI_DISP, AMI::AMI_DISP_BORDER, changecolor);
    }
  }

  void
  lib_display::change_fill(DISPLAY_COLORS c)
  {
    if(os_->kernel_supported(AMI::AMI_DISP, AMI::AMI_DISP_FILL)){
        sst_message::ptr changecolor = sst_message::construct("dummy",
            timestamp(0), sst_message::SST, c);
        os_->execute_kernel(AMI::AMI_DISP, AMI::AMI_DISP_FILL, changecolor);
    }
  }

  } } //end of namespace sstmac
