/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_DUMPIALLOCATION_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_DUMPIALLOCATION_H_INCLUDED

#include <sstmac/software/services/launch/allocation/allocationstrategy.h>
#include <sstmac/software/skeletons/undumpi/dumpimeta.h>

#include <sstmac/hardware/network/topology/topology.h>

namespace sstmac
{
  namespace sw
  {
    class dumpiallocation : public allocationstrategy
    {
    protected:
      dumpiallocation(std::string mapname, sstmac::hw::topology::ptr top);

    public:
      typedef boost::intrusive_ptr<dumpiallocation> ptr;

      static ptr
      construct(std::string mapname, sstmac::hw::topology::ptr top)
      {
        return ptr(new dumpiallocation(mapname, top));
      }

      static ptr
      construct(sim_parameters::ptr params, sstmac::hw::topology::ptr top)
      {
        return construct(params->get_param("launch_dumpi_mapname"), top);
      }

      /**
       * This function reads in a map file for mapping host names to network coordinates.
       * @param here string used for debugging / exception throwing
       * @param mapfile name of the map file
       * @param hostmap map in which host mappings are placed
       */
      static void
      read_map_file(const std::string &here, const std::string &mapfile,
          std::map<std::string, std::vector<int> >&hostmap);

      virtual void
      allocate(int nnode_requested,
          std::set<nodeaddress::ptr, nodeaddress::nodeaddresscomp> &allocation);

      virtual
      ~dumpiallocation() throw() {}

      static boost::unordered_map<std::string, nodeaddress::ptr> hostnamemap_;
    protected:
      std::string mapfile_;
      sstmac::hw::topology::ptr top_;
    };

  }
}

#endif /* DUMPIALLOCATION_H_ */
