/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_LAUNCH_LAUNCH_MESSAGE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_LAUNCH_LAUNCH_MESSAGE_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>

#include <sstmac/software/services/launch/launchinfo.h>

namespace sstmac
{
  namespace sw
  {

    class launch_message : public sst_message
    {

    public:
      typedef boost::intrusive_ptr<launch_message> ptr;

      static ptr
      construct(launchinfo::ptr &i)
      {
        return ptr(new launch_message(i));
      }

      /**
       * Stringifier
       * @return String description
       */
      virtual std::string
      to_string() const
      {
        return "launch_message";
      }

      launchinfo::ptr
      get_info()
      {
        return info_;
      }

    protected:
      launch_message(launchinfo::ptr &i) :
          sst_message("launcher", i->start_, sst_message::LAUNCH, 0), info_(i)
      {

      }

      launchinfo::ptr info_;
    };

  }
}

#endif
