/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_LAUNCHER_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_LAUNCHER_H_INCLUDED

//#include <sstmac/software/ami/environment.h>
//#include <sstmac/backends/native/nodelist.h>
#include <sstmac/common/timestamp.h>
#include <string>
#include <sstmac/common/ptr_type.h>
#include <sstmac/common/debug.h>
#include <sstmac/software/services/service.h>

#include <vector>

namespace sstmac { namespace sw
  {



  /**
   * A launcher that can be cooperatively scheduled by a very naive scheduler.
   */
  class launcher : public service
  {

  public:
    virtual std::string to_string() const{
            return "launcher";
          }

    typedef boost::intrusive_ptr<launcher> ptr;

    /// Hasta la vista.
    virtual ~launcher() throw ()
    {
    }


    virtual std::string
    get_service_name();


    virtual void complete() {
    	is_completed_ = true;
    }

    /// Test whether this launcher has completed.
    virtual bool completed() const {
    	return is_completed_;
    }

  protected:
    launcher()  : is_completed_(false)  {
    	if (!dbg_.is_real()) {
    		dbg_ = debug("<debug> launcher | launch");
    		logger::register_logger(&dbg_);
    	}
    }

    static debug dbg_;

    bool is_completed_;
  };


  } } // end of namespace sstmac

#endif
