/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/skeletons/mini/minimd/lib_compute_minimd.h>

namespace sstmac { namespace sw
{
  using namespace std;
  
  

  std::string
  lib_compute_minimd::get_lib_name()
  {
    std::stringstream ss;
    ss << "minimdcomputelib" << id_.app_.id << "-" << id_.task_.id;
    return ss.str();
  }

  lib_compute_minimd::lib_compute_minimd(softwareid id) :
    lib_compute_time(id)

  {

    funcs_["create_atoms"] = timestamp(1E-7);
    funcs_["create_velocity"] = timestamp(1E-7);
    funcs_["output"] = timestamp(1E-7);
    funcs_["create_atoms"] = timestamp(1E-7);
    funcs_["create_atoms"] = timestamp(1E-7);

    funcs_["Neighbor::build"] = timestamp(1E-7);
    funcs_["Thermo::temperature"] = timestamp(1E-7);
    funcs_["Thermo::energy"] = timestamp(1E-7);
    funcs_["Force::compute"] = timestamp(1E-7);
    funcs_["Thermo::pressure"] = timestamp(1E-7);
    funcs_["Thermo::compute"] = timestamp(1E-7);
    funcs_["Integrate::run"] = timestamp(1E-7);

  }

  lib_compute_minimd::~lib_compute_minimd()
  {

  }


  //
  // Allocate an object of this type.
  //
  lib_compute_minimd::ptr
  lib_compute_minimd::construct(softwareid it)
  {
    lib_compute_minimd::ptr retval(new lib_compute_minimd(it));

    return retval;
  }

  void
  lib_compute_minimd::compute(std::string func, int index)
  {
    if (funcs_.find(func) == funcs_.end())
      {
        std::cout << "function: " << func << endl;
        throw libraryerror("lib_compute_minimd: cannot find function");
      }

    lib_compute_time::compute(funcs_[func]);
  }

  int
  lib_compute_minimd::get_int(std::string func, int index)
  {

    return 1;
  }

} } //end of namespace sstmac
