/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_THERMO_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_THERMO_H_INCLUDED

#include <sstmac/software/skeletons/mini/minimd/minimd.h>
#include <sstmac/software/skeletons/mini/minimd/minimd-force.h>
#include <sstmac/software/skeletons/mini/minimd/minimd-atom.h>

namespace sstmac { namespace sw {

  

  /**
   * Pretend to do thermodynamics.
   */
  class minimd::thermo  : public ptr_type {
  private:
    lib_compute_minimd::ptr execution_;
    mpiapi::ptr mpi_;

  public:
    virtual std::string to_string() const{
            return "thermo";
          }

    typedef boost::intrusive_ptr<thermo> ptr;

    int nstat;
    int ntimes;

    void init(const lib_compute_minimd::ptr& exe,
              const mpiapi::ptr& mpi)
    {
      execution_ = exe;
      mpi_ = mpi;
    }

    void compute(int iflag, const atom::ptr& atm, const boost::intrusive_ptr<neighbor>& nbr,
                 const force::ptr& frc)
    {
      if (iflag > 0 && iflag % nstat) return;
      if (iflag == -1 && nstat > 0 && ntimes % nstat == 0) return;      

      temperature(atm);
      energy(atm);
      frc->compute(atm, nbr);
      pressure(atm);
      execution_->compute("Thermo::compute");
    }
    
    void energy(const atom::ptr&) {
      execution_->compute("Thermo::energy");
      mpi_->allreduce(1, mpitype::mpi_double, mpiop::sum, mpi_->comm_world());
    }

    void temperature(const atom::ptr&) {
      execution_->compute("Thermo::temperature");
      mpi_->allreduce(1, mpitype::mpi_double, mpiop::sum, mpi_->comm_world());
    }

    void pressure(const atom::ptr&) {
      execution_->compute("Thermo::pressure");
      mpi_->allreduce(1, mpitype::mpi_double, mpiop::sum, mpi_->comm_world()); 
    }
  };

} } // end of namespace sstmac

#endif
