/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/skeletons/skeletonfactory.h>

#include <sstmac/software/process/softwareid.h>

#include <sstmac/software/skeletons/test/app_helloworld.h>
#include <sstmac/software/skeletons/test/mpipingpong.h>
#include <sstmac/software/skeletons/test/mpipairwise.h>
#include <sstmac/software/skeletons/test/mpi_testall.h>
#include <sstmac/software/skeletons/mini/mpi3d/mpi3d.h>
#include <sstmac/software/skeletons/mini/minimd/minimd.h>
#include <sstmac/software/skeletons/mini/actormatmul/actormatmul.h>
#include <sstmac/software/skeletons/mini/systolicmxm/systolicmxm.h>
#include <sstmac/software/skeletons/osu/osubw.h>
#include <sstmac/software/skeletons/osu/osulat.h>
#include <sstmac/software/skeletons/undumpi/parsedumpi.h>

#include <sstmac/software/skeletons/test/test_compute.h>
#include <sstmac/software/skeletons/examples/example_shared_memory.h>

namespace sstmac { namespace sw
{

  std::map<std::string, app::ptr > skeletonfactory::apps;
  bool skeletonfactory::initialized_ = false;

  void
  skeletonfactory::init_skeletons()
  {
    if (!initialized_)
      { //only do it once
        appid dumma(-1);
        taskid dummt(-1);
        softwareid dumms(dumma, dummt);

        skeletonfactory::apps["hello"] =
          sstmac::sw::app_helloworld::construct(dumms);

        skeletonfactory::apps["MPI_pingpong"] =
          sstmac::sw::mpipingpong::construct(dumms);

        skeletonfactory::apps["MPI_pairwise"] =
                  sstmac::sw::mpipairwise::construct(dumms);

        skeletonfactory::apps["MPI_testall"] =
          sstmac::sw::mpi_testall::construct(dumms);

        skeletonfactory::apps["test_compute"] =
          sstmac::sw::test_compute::construct(dumms);

        skeletonfactory::apps["example_shared_memory"] =
            sstmac::sw::example_shared_memory::construct(dumms);

        //mini's
        skeletonfactory::apps["mpi3d"] =
          sstmac::sw::mpi3d::construct(dumms);

        skeletonfactory::apps["minimd"] =
          sstmac::sw::minimd::construct(dumms);

        skeletonfactory::apps["actormatmul"] =
          sstmac::sw::actormatmul::construct(dumms);

        skeletonfactory::apps["systolicmxm"] =
          sstmac::sw::systolicmxm::construct(dumms,4,4,4,100,100,100,
                                             4,4,1,1,true,1);
        
        //benchmarks
        skeletonfactory::apps["OSU_bandwidth"] =
          sstmac::sw::osubw::construct(dumms);

        skeletonfactory::apps["OSU_latency"] =
          sstmac::sw::osulat::construct(dumms);


        skeletonfactory::apps["parsedumpi"] =
            sstmac::sw::parsedumpi::construct(dumms);

        initialized_ = true;
      }
  }

  void
  skeletonfactory::print_factory(){
    std::cout << "Contents of skeleton factory are: " << std::endl;

    std::map<std::string, app::ptr>::iterator it, end = apps.end();
    for(it = apps.begin(); it != end; it++){
        std::cout << it->first << std::endl;
    }
  }

} } //end of namespace sstmac
