/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_TEST_MPI_TESTALL_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_TEST_MPI_TESTALL_H_INCLUDED

#include <sstmac/software/libraries/mpi/mpiapp.h>
#include <list>

namespace sstmac { namespace sw
{

  class mpi_testall : public mpiapp
  {
  protected:

    static int errors_;

    mpi_testall(softwareid id) :
      mpiapp(id)
    {
    }

    void test_sendrecv();
    void test_barrier();
    void test_asynch();
    void test_bcast();
    void test_scatter();
    void test_gather();
    void test_allgather();
    void test_reduce();
    void test_allreduce();
    void test_alltoall();
    void test_comms();
    void test_wait();
    void test_reducescatter();
    void test_probe();
    void test_persistent();

    bool stop_at_errors_;

  public:
    typedef boost::intrusive_ptr<mpi_testall> ptr;

    /// Hello.
    static mpi_testall::ptr
    construct(softwareid id)
    {
      mpi_testall::ptr ret(new mpi_testall(id));
      return ret;
    }

    virtual void
    consume_params(const sim_parameters::ptr &params);

    /// Get a copy.
    virtual app::ptr
    clone(softwareid id){
      return construct(id);
    }

    virtual
    ~mpi_testall();

    /// Go.
    void
    skeleton_main();

  };

} } //end of namespace sstmac
#endif
